@@ -7,6 +7,8 @@
 !                            ʻ̤ɬפ뤿. 
 !      2002/06/10  ݹ  ݥ뼧춭롼ɲ
 !      2002/11/10  ݹ  ¶֤Ǥζ롼ɲ
+!      2002/11/24  ݹ  ӥշϤ
+!                            ץ饷ղ򤭥롼ɲ
 !
 !
 !      ǡ index
@@ -61,13 +63,13 @@ module wt_module
   public wt_Potential2vector
   public wt_VGradV
 
-  public wt_Boundaries, wt_TorBoundaries
+  public wt_Boundaries, wt_TorBoundaries, wz_LaplaPol2Pol_wz
   public wt_TormagBoundaries, wt_PolmagBoundaries
 
   public wt_BoundariesTau, wt_TorBoundariesTau
   public wt_TormagBoundariesTau, wt_PolmagBoundariesTau
 
-  public wt_BoundariesGrid, wt_TorBoundariesGrid, wz_LaplaPol2Pol_wz
+  public wt_BoundariesGrid, wt_TorBoundariesGrid, wt_LaplaPol2PolGrid_wt
   public wt_TormagBoundariesGrid, wt_PolmagBoundariesGrid
 
   interface wt_Boundaries
@@ -688,7 +690,7 @@ module wt_module
          case ('FF')
             rigid1 = .FALSE. ; rigid2 = .FALSE.
          case default
-            call msgdmp('E','wt_TorBoundaries','B.C. not supported')
+            call msgdmp('E','wt_TorBoundariesTau','B.C. not supported')
          end select
       endif
 
@@ -771,7 +773,7 @@ module wt_module
          case ('FF')
             rigid1 = .FALSE. ; rigid2 = .FALSE.
          case default
-            call msgdmp('E','wt_TorBoundaries','B.C. not supported')
+            call msgdmp('E','wt_TorBoundariesGrid','B.C. not supported')
          end select
       endif
 
@@ -875,6 +877,11 @@ module wt_module
       if ( first ) then
          first = .false.
 
+         if ( lm /= km ) then
+            call msgdmp('E','wz_LaplaPol2Pol_wz', &
+             'Chebyshev truncation and number of grid points should be same.')
+         endif
+
          allocate(alu((nm+1)*(nm+1),0:km,0:km),kp((nm+1)*(nm+1),0:km))
 
          do k=0,km
@@ -927,6 +934,125 @@ module wt_module
 
     end function wz_LaplaPol2Pol_wz
 
+    function wt_LaplaPol2PolGrid_wt(wt,cond)    ! ^2 ݥ
+                                                ! ǥեȤǴ
+      ! Chebyshev-Collocation ˡˤ׻
+      !  f=\nabla^2\phi Ϳƥݥݥƥ󥷥 \phi .
+      !    \nabla^2 \phi = f
+      !    \phi = const. at boundaries.
+      ! Ǵ
+      !    \DP{\phi}{r} = 0 at boundaries
+      ! Ϥʤ
+      !    \DP[2]{\phi}{r} = 0 at boundaries
+
+      !  : ǽ˸ƤФȤζǰʸ׻(׻ѹ)
+
+      real(8), dimension((nm+1)*(nm+1),0:lm),intent(in)  :: wt
+              ! Ϣ^2ʬ
+
+      real(8), dimension((nm+1)*(nm+1),0:lm)             :: wt_LaplaPol2PolGrid_wt
+              ! ϥݥݥƥ󥷥ʬ
+
+      character(len=2), intent(in), optional  :: cond
+              ! 凉å. ά 'RR'
+              !   RR : ξüǴ
+              !   RF,FR : Ǵ/Ϥʤ
+              !   FF : ξüϤʤ
+
+      real(8), dimension(:,:,:), allocatable  :: alu
+      integer, dimension(:,:), allocatable    :: kp
+
+      real(8), dimension((nm+1)*(nm+1),0:km)  :: wz_work
+      real(8), dimension((nm+1)*(nm+1),0:lm)  :: wt_work
+      real(8), dimension(0:lm,0:lm)           :: tt_I
+      real(8), dimension(0:lm,0:km)           :: tz_work
+      logical                                 :: rigid1, rigid2   ! 
+
+      logical :: first = .true.
+      integer :: l,n
+      save    :: alu, kp, first
+
+      if (.not. present(cond)) then
+         rigid1=.TRUE. ; rigid2=.TRUE.
+      else
+         select case (cond)
+         case ('RR')
+            rigid1 = .TRUE.  ; rigid2 = .TRUE.
+         case ('RF')
+            rigid1 = .TRUE.  ; rigid2 = .FALSE.
+         case ('FR')
+            rigid1 = .FALSE. ; rigid2 = .TRUE.
+         case ('FF')
+            rigid1 = .FALSE. ; rigid2 = .FALSE.
+         case default
+            call msgdmp('E','wt_LaplaPol2PolGrid_wt','B.C. not supported')
+         end select
+      endif
+
+      if ( first ) then
+         first = .false.
+
+         if ( lm /= km ) then
+            call msgdmp('E','wt_LaplaPol2PolGrid_wt', &
+             'Chebyshev truncation and number of grid points should be same.')
+         endif
+
+         allocate(alu((nm+1)*(nm+1),0:km,0:lm),kp((nm+1)*(nm+1),0:lm))
+
+         do l=0,lm
+            wt_work = 0 ; wt_work(:,l) = 1
+
+            ! ƿʿȿ˴ؤΩμ
+            alu(:,:,l) = wz_wt(wt_Lapla_wt(wt_work))
+         enddo
+
+         ! ưŪ. ή϶ǰ
+         tt_I = 0.0
+         do l=0,lm
+            tt_I(l,l)=1
+         enddo
+
+         ! ŵƳ
+         tz_work = az_at(tt_I)
+
+         do n=1,(nm+1)*(nm+1)
+            alu(n,0,:)  = tz_work(:,0)
+            alu(n,km,:) = tz_work(:,km)
+         enddo
+
+         ! ϳŪǴ 
+         if ( rigid1 ) then
+            tz_work=az_at(at_Dr_at(tt_I))
+         else
+            tz_work=az_at(at_Dr_at(at_Dr_at(tt_I)))
+         endif
+         do n=1,(nm+1)*(nm+1)
+            alu(n,1,:) = tz_work(:,0)
+         enddo
+
+         ! ϳŪǴ 
+         if ( rigid2 ) then
+            tz_work=az_at(at_Dr_at(tt_I))
+         else
+            tz_work=az_at(at_Dr_at(at_Dr_at(tt_I)))
+         endif
+         do n=1,(nm+1)*(nm+1)
+            alu(n,km-1,:) = tz_work(:,km)
+         enddo
+
+         call ludecomp(alu,kp)
+      endif
+
+      wz_work         = wz_wt(wt)
+      wz_work(:,1)    = 0               ! ϳŪ
+      wz_work(:,km-1) = 0               ! ϳŪ
+      wz_work(:,0)    = 0               ! ưŪ
+      wz_work(:,km)   = 0               ! ưŪ 
+
+      wt_LaplaPol2PolGrid_wt = lusolve(alu,kp,wz_work)
+
+    end function wt_LaplaPol2PolGrid_wt
+
     subroutine wt_TormagBoundariesTau(wt_TOR) ! ȥݥƥ󥷥붭
       ! Chebyshev ֤ǤζŬ
       ! ȥݥƥ󥷥 \psi ζ(ŵƳ)
